import { ComponentFixture, TestBed } from '@angular/core/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { AppComponent } from './app.component';
import { AuthService } from '../common/auth/auth.service';
import { Mock, mock, existsAsHtmlElement, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ApiRootService } from '@alfa-client/api-root-shared';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createEmptyStateResource, createStateResource } from '@alfa-client/tech-shared';
import { of } from 'rxjs';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { MockComponent } from 'ng-mocks';
import { UserProfileButtonContainerComponent } from '../common/user-profile-button-container/user-profile.button-container.component';

describe('AppComponent', () => {
  let component: AppComponent;
  let fixture: ComponentFixture<AppComponent>;

  const adminHeader: string = getDataTestIdOf('admin-header');
  const buildVersion: string = getDataTestIdOf('build-version');
  const userProfileButton: string = getDataTestIdOf('user-profile-button');

  const authService: Mock<AuthService> = mock(AuthService);
  const apiRootService: Mock<ApiRootService> = mock(ApiRootService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [AppComponent, MockComponent(UserProfileButtonContainerComponent)],
      imports: [RouterTestingModule],
      providers: [
        {
          provide: AuthService,
          useValue: authService,
        },
        {
          provide: ApiRootService,
          useValue: apiRootService,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(AppComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it(`should have as title 'admin'`, () => {
    const appTitle: string = fixture.componentInstance.title;

    expect(appTitle).toEqual('admin');
  });

  describe('ngOnInit', () => {
    it('should call authService login', () => {
      component.ngOnInit();

      expect(authService.login).toHaveBeenCalled();
    });

    it('should call doAfterLoggedIn', async () => {
      authService.login.mockImplementation(() => Promise.resolve());
      component.doAfterLoggedIn = jest.fn();

      await component.ngOnInit();

      expect(component.doAfterLoggedIn).toHaveBeenCalled();
    });

    describe('do after logged in', () => {
      it('should call apiRootService to getApiRoot', () => {
        component.doAfterLoggedIn();

        expect(apiRootService.getApiRoot).toHaveBeenCalled();
      });
    });
  });

  it('show not show header if apiRoot is not loaded', () => {
    component.apiRoot$ = of(createEmptyStateResource());

    notExistsAsHtmlElement(fixture, adminHeader);
  });

  describe('user profile button', () => {
    beforeEach(() => {
      component.apiRoot$ = of(createStateResource(createApiRootResource()));
    });

    it('should show if apiRoot exists', () => {
      fixture.detectChanges();

      existsAsHtmlElement(fixture, userProfileButton);
    });
  });

  describe('build version', () => {
    beforeEach(() => {
      component.apiRoot$ = of(createStateResource(createApiRootResource()));
    });

    it('should show after apiRoot loaded', () => {
      fixture.detectChanges();

      existsAsHtmlElement(fixture, buildVersion);
    });
  });
});
