import { ApiRootModule } from '@alfa-client/api-root-shared';
import { ENVIRONMENT_CONFIG, EnvironmentModule } from '@alfa-client/environment-shared';
import { CommonModule } from '@angular/common';
import { HttpClientModule } from '@angular/common/http';
import { APP_INITIALIZER, NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { RouterModule } from '@angular/router';
import { EffectsModule } from '@ngrx/effects';
import { StoreRouterConnectingModule } from '@ngrx/router-store';
import { StoreModule } from '@ngrx/store';
import { StoreDevtoolsModule } from '@ngrx/store-devtools';
import { TestbtnComponent } from 'design-system';
import { KeycloakAngularModule, KeycloakService } from 'keycloak-angular';
import { environment } from '../environments/environment';
import { AppComponent } from './app.component';
import { initializeKeycloak } from './app.config';
import { appRoutes } from './app.routes';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { UserProfileButtonContainerComponent } from '../common/user-profile-button-container/user-profile.button-container.component';

@NgModule({
  declarations: [AppComponent, UserProfileButtonContainerComponent],
  imports: [
    CommonModule,
    TestbtnComponent,
    RouterModule.forRoot(appRoutes),
    BrowserModule,
    BrowserAnimationsModule,
    HttpClientModule,
    KeycloakAngularModule,
    ApiRootModule,
    EnvironmentModule,
    environment.production ? [] : StoreDevtoolsModule.instrument(),
    StoreModule.forRoot({}),
    EffectsModule.forRoot(),
    StoreRouterConnectingModule.forRoot(),
    FormsModule,
    ReactiveFormsModule,
  ],
  providers: [
    {
      provide: APP_INITIALIZER,
      useFactory: initializeKeycloak,
      multi: true,
      deps: [KeycloakService, ENVIRONMENT_CONFIG],
    },
  ],
  bootstrap: [AppComponent],
})
export class AppModule {}
