/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { convertToDataTestId } from '../../support/tech.util';
import { UserProfileE2EComponent } from '../user-profile/user-profile.component.e2e';
import { AttachmentContainerE2EComponent } from '../attachment/attachment.e2e.component';

export class KommentareInVorgangE2EComponent {
  readonly locatorHinzufuegenButton: string = 'create-kommentar';
  readonly locatorFormular: string = 'kommentar-formular';
  readonly locatorFormularSpeichernButton: string = 'add-button';
  readonly locatorFormularAbbrechenButton: string = 'cancel-button';
  readonly locatorKommentarList: string = 'kommentar-list-in-vorgang';
  readonly locatorTextInput: string = 'Kommentar-textarea-input';
  readonly locatorTextError: string = 'Kommentar-textarea-error';

  private readonly attachmentContainer: AttachmentContainerE2EComponent =
    new AttachmentContainerE2EComponent();

  readonly locatorRoot: string = 'kommentar-container-in-vorgang';

  public getRoot() {
    return cy.getTestElement(this.locatorRoot);
  }

  public getFormular() {
    return cy.getTestElement(this.locatorFormular);
  }

  public getHinzufuegenButton() {
    return cy.getTestElement(this.locatorHinzufuegenButton);
  }

  public getFormularSpeichernButton() {
    return cy.getTestElement(this.locatorFormularSpeichernButton);
  }

  public getFormularAbbrechenButton() {
    return cy.getTestElement(this.locatorFormularAbbrechenButton);
  }

  public getKommentarList() {
    return cy.getTestElement(this.locatorKommentarList);
  }

  public getTextInput() {
    return cy.getTestElement(this.locatorTextInput);
  }

  public getTextError() {
    return cy.getTestElement(this.locatorTextError);
  }

  public getKommentar(text: string): KommentarInVorgangE2E {
    return new KommentarInVorgangE2E(text);
  }

  public getAttachmentContainer(): AttachmentContainerE2EComponent {
    return this.attachmentContainer;
  }
}

export class KommentarInVorgangE2E {
  //TODO: Rename to KommentarInVorgangE2EComponent

  private readonly locatorCreatedAt: string = 'kommentar-created-at';

  private readonly locatorRoot: string;

  constructor(private text: string) {
    this.locatorRoot = 'kommentar-item-' + convertToDataTestId(this.text);
  }

  public getRoot() {
    return cy.getTestElement(this.locatorRoot);
  }

  public getUserProfile(): UserProfileE2EComponent {
    return new UserProfileE2EComponent(this.locatorRoot);
  }

  public getCreatedAt() {
    return cy.getTestElement(this.locatorCreatedAt);
  }
}
