/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { VorgangDetailHeaderE2EComponent } from '../../../components/vorgang/vorgang-detail-header.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangSubnavigationE2EComponent } from '../../../components/vorgang/vorgang-subnavigation';
import { HttpMethodE2E } from '../../../model/util';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import {
  dropCollections,
  interceptWithResponse,
  waitOfInterceptor,
} from '../../../support/cypress-helper';
import { contains, exist, notExist } from '../../../support/cypress.util';
import { loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgang } from '../../../support/vorgang-util';

//FIXME Stand 07.09 => Bei einem 401 wird in der aktuellen Implementierung zum Keycloak weitergeleitet.
//Sofern das die richtige Implementierung ist, muss der Test angepasst werden, andernfalls die Implementierung
describe('Interceptor unauthorized', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangHeader: VorgangDetailHeaderE2EComponent = vorgangPage.getVorgangDetailHeader();
  const subnavigation: VorgangSubnavigationE2EComponent = vorgangPage.getSubnavigation();

  const vorgang: VorgangE2E = createVorgang();

  before(() => {
    initVorgang(vorgang);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('click on vorgang', () => {
    const BASE_PATH: string = '*/vorgangs/';
    const statusCode: number = 401;
    const interceptorResponseUnauthorized: string = 'getVorgangWithEingangUnauthorized';

    it('should reload page', () => {
      interceptWithResponse(HttpMethodE2E.GET, BASE_PATH + vorgang._id.$oid, { statusCode }).as(
        interceptorResponseUnauthorized,
      );

      vorgangList.getListItem(vorgang.name).getRoot().click({ force: true });
      waitForSpinnerToDisappear();

      waitOfInterceptor(interceptorResponseUnauthorized).then((interception) => {
        assert(interception.response.statusCode, statusCode.toString());
        notExist(vorgangHeader.getRoot());
      });
    });

    it('should land on Keycloak login page', () => {
      exist(cy.get('#kc-login'));
    });
  });
});
