import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangFormularButtonsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { VorgangSubnavigationE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-subnavigation';
import { VorgangAttachedItemE2E } from 'apps/alfa-e2e/src/model/vorgang-attached-item';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { contains, exist, haveText, notExist } from '../../../support/cypress.util';
import { loginAsSabine } from '../../../support/user-util';
import {
  createLoeschAnforderungAttachedItem,
  initVorgangAttachedItem,
} from '../../../support/vorgang-attached-item-util';
import { buildVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('LoeschAnforderung zurücknehmen', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangFormularButtons: VorgangFormularButtonsE2EComponent =
    vorgangPage.getFormularButtons();
  const subnavigation: VorgangSubnavigationE2EComponent = vorgangPage.getSubnavigation();

  const vorgang: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'DoLoeschenAnforderungZuruecknehmen'),
    status: VorgangStatusE2E.ZU_LOESCHEN,
  };
  const vorgangLoeschAnforderung: VorgangAttachedItemE2E = createLoeschAnforderungAttachedItem(
    objectIds[1],
    vorgang._id.$oid,
  );

  const vorgangRevoke: VorgangE2E = {
    ...buildVorgang(objectIds[2], 'DoRevokeLoeschenAnforderungZuruecknehmen'),
    status: VorgangStatusE2E.ZU_LOESCHEN,
  };
  const vorgangRevokeLoeschAnforderung: VorgangAttachedItemE2E =
    createLoeschAnforderungAttachedItem(objectIds[3], vorgangRevoke._id.$oid);

  before(() => {
    initVorgaenge([vorgang, vorgangRevoke]);
    initVorgangAttachedItem([vorgangLoeschAnforderung, vorgangRevokeLoeschAnforderung]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('zuruecknehmen by button', () => {
    describe('navigate', () => {
      it('should open Vorgang on item click', () => {
        vorgangList.getListItem(vorgang.name).getRoot().click();
        waitForSpinnerToDisappear();

        exist(vorgangPage.getVorgangDetailHeader().getRoot());
      });
    });

    describe('ui elements', () => {
      it('should have status Zu Löschen', () => {
        haveText(
          vorgangPage.getVorgangDetailHeader().getStatus(),
          vorgangStatusLabelE2E[vorgang.status],
        );
      });

      it('should show zuruecknehmen button', () => {
        exist(vorgangFormularButtons.getLoeschAnforderungZuruecknehmenButton());
      });
    });

    describe('on click on zuruecknehmen button', () => {
      before(() => {
        vorgangFormularButtons.getLoeschAnforderungZuruecknehmenButton().click();
      });

      it('should show snackbar', () => {
        contains(
          mainPage.getSnackBar().getMessage(),
          'Für den Vorgang wurde die Löschanforderung zurückgenommen.',
        );
        exist(mainPage.getSnackBar().getCloseButton());
        exist(mainPage.getSnackBar().getRevokeButton());
      });

      it('should close snackbar on close button', () => {
        mainPage.getSnackBar().getCloseButton().click();

        notExist(mainPage.getSnackBar().getMessage());
      });

      it('should change status', () => {
        haveText(
          vorgangPage.getVorgangDetailHeader().getStatus(),
          vorgangStatusLabelE2E[VorgangStatusE2E.ABGESCHLOSSEN],
        );
      });

      it('should hide zuruecknehmen button', () => {
        notExist(vorgangFormularButtons.getLoeschAnforderungZuruecknehmenButton());
      });

      it('should show loschenAnfordern button', () => {
        exist(vorgangFormularButtons.getLoeschenAnfordernButton());
      });

      it('should navigate to list on back to vorgang list', () => {
        vorgangPage.getSubnavigation().getBackButton().click();
        waitForSpinnerToDisappear();

        exist(vorgangList.getRoot());
      });

      it('should have status Abgeschlossen', () => {
        haveText(
          vorgangList.getListItem(vorgang.name).getStatus(),
          vorgangStatusLabelE2E[VorgangStatusE2E.ABGESCHLOSSEN],
        );
      });
    });
  });

  describe('zuruecknehmen und rückgängig machen by icon button', () => {
    describe('navigate', () => {
      it('should open Vorgang on item click', () => {
        vorgangList.getListItem(vorgangRevoke.name).getRoot().click();
        waitForSpinnerToDisappear();

        exist(vorgangPage.getVorgangDetailHeader().getRoot());
      });
    });

    describe('ui elements', () => {
      it('should have status Zu Löschen', () => {
        haveText(
          vorgangPage.getVorgangDetailHeader().getStatus(),
          vorgangStatusLabelE2E[vorgang.status],
        );
      });

      it('should show zuruecknehmen icon button', () => {
        exist(subnavigation.getLoeschAnforderungZuruecknehmenIconButton());
      });
    });

    describe('on click on zuruecknehmen button', () => {
      before(() => {
        subnavigation.getLoeschAnforderungZuruecknehmenIconButton().click();
      });

      it('should show snackbar', () => {
        contains(
          mainPage.getSnackBar().getMessage(),
          'Für den Vorgang wurde die Löschanforderung zurückgenommen.',
        );
        exist(mainPage.getSnackBar().getCloseButton());
        exist(mainPage.getSnackBar().getRevokeButton());
      });

      it('should show progress bar on revoke button', () => {
        mainPage.getSnackBar().getRevokeButton().click();

        exist(vorgangPage.getProgressBar());
        waitForSpinnerToDisappear();
      });

      it('should keep status', () => {
        haveText(
          vorgangPage.getVorgangDetailHeader().getStatus(),
          vorgangStatusLabelE2E[VorgangStatusE2E.ZU_LOESCHEN],
        );
      });

      it('should show zuruecknehmen button', () => {
        exist(subnavigation.getLoeschAnforderungZuruecknehmenIconButton());
      });

      it('should hide loschenAnfordern button', () => {
        notExist(vorgangFormularButtons.getLoeschenAnfordernButton());
      });

      it('should navigate to list on back to vorgang list', () => {
        vorgangPage.getSubnavigation().getBackButton().click();
        waitForSpinnerToDisappear();

        exist(vorgangList.getRoot());
      });

      it('should have status ZU_LOESCHEN', () => {
        haveText(
          vorgangList.getListItem(vorgangRevoke.name).getStatus(),
          vorgangStatusLabelE2E[VorgangStatusE2E.ZU_LOESCHEN],
        );
      });
    });
  });
});
