/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import { generate12ByteId } from '../../../support/tech.util';
import { loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgaenge } from '../../../support/vorgang-util';

describe('Postfach Nachricht reply button', () => {
  const vorgangList: VorgangListE2EComponent = new MainPage().getVorgangList();
  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangWithOsiServiceKonto: VorgangE2E = createVorgang();
  const vorgangWithBayernIdServiceKonto: VorgangE2E = {
    ...createVorgang(),
    _id: {
      $oid: generate12ByteId(),
    },
    name: 'BayernID Vorgang',
    header: {
      serviceKonto: {
        type: 'BayernId',
        postfachAddress: [
          {
            type: 1,
            identifier: new Map([
              ['postfachId', new Object('04d39269-81c5-4838-8b73-08d9567f06d7')],
            ]),
          },
        ],
      },
    },
  };

  before(() => {
    initVorgaenge([vorgangWithOsiServiceKonto, vorgangWithBayernIdServiceKonto]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('navigate to vorgang detail with OSI service konto', () => {
    it('should open vorgang detail', () => {
      vorgangList.getListItem(vorgangWithOsiServiceKonto.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should open new message editor', () => {
      vorgangPage.getPostfachMailcontainer().getCreateButtonWithText().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getPostfachMailFormular().getSendButton());
    });

    it('should have reply option available', () => {
      exist(vorgangPage.getPostfachMailFormular().getReplyOption().getRoot());
    });
  });

  describe('navigate to vorgang detail with BayernID service konto', () => {
    it('should navigate to vorgang list page', () => {
      vorgangPage.getSubnavigation().getBackButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangList.getRoot());
    });

    it('should open vorgang detail', () => {
      vorgangList.getListItem(vorgangWithBayernIdServiceKonto.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should open new message editor', () => {
      vorgangPage.getPostfachMailcontainer().getCreateButtonWithText().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getPostfachMailFormular().getSendButton());
    });

    it('should not have reply option available', () => {
      notExist(vorgangPage.getPostfachMailFormular().getReplyOption().getRoot());
    });
  });
});
