/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { PostfachMailE2EComponent } from '../../../components/postfach/postfach-mail.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { HttpMethodE2E, MessageCodeE2E } from '../../../model/util';
import { VorgangE2E } from '../../../model/vorgang';
import {
  DirectionE2E,
  PostfachMailItemE2E,
  VorgangAttachedItemE2E,
} from '../../../model/vorgang-attached-item';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import {
  dropCollections,
  interceptWithResponse,
  waitOfInterceptor,
} from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import {
  getUserManagerUserEmil,
  getUserManagerUserPeter,
  getUserManagerUserSabine,
  initUsermanagerUsers,
  loginAsSabine,
} from '../../../support/user-util';
import {
  createPostfachNachrichtAttachedItem,
  createPostfachNachrichtReplyItem,
  initVorgangAttachedItem,
} from '../../../support/vorgang-attached-item-util';
import { createVorgang, initVorgang, objectIds } from '../../../support/vorgang-util';

describe('Postfach nachricht user profile on backend error', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const postfachNachrichtenContainer: PostfachMailE2EComponent =
    vorgangPage.getPostfachMailcontainer();

  const vorgang: VorgangE2E = createVorgang();

  const postfachMailItem: PostfachMailItemE2E = {
    ...createPostfachNachrichtReplyItem(),
    createdBy: '6287e7507433b51bfc87cfe9',
    direction: DirectionE2E.OUT,
    sentAt: '2022-12-02T15:00:00.790Z[UTC]',
    sentSuccessful: true,
  };
  const vorgangAttachedItem: VorgangAttachedItemE2E = {
    ...createPostfachNachrichtAttachedItem(objectIds[0], vorgang._id.$oid),
    item: postfachMailItem,
  };

  before(() => {
    initVorgang(vorgang);
    initVorgangAttachedItem([vorgangAttachedItem]);
    initUsermanagerUsers([
      getUserManagerUserSabine(),
      getUserManagerUserPeter(),
      getUserManagerUserEmil(),
    ]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('on status 404', () => {
    it('should navigate to vorgang detail', () => {
      vorgangList.getListItem(vorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
      waitForSpinnerToDisappear();
    });

    it('should show error not found icon', () => {
      exist(
        postfachNachrichtenContainer
          .getListItem(postfachMailItem.subject)
          .getUserProfile()
          .getIconContainer()
          .getErrorResourceNotFoundIcon(),
      );
    });

    it('should hide user profile name', () => {
      notExist(
        postfachNachrichtenContainer
          .getListItem(postfachMailItem.subject)
          .getUserProfile()
          .getName(),
      );
    });

    it('should navigate back to list', () => {
      vorgangPage.getSubnavigation().getBackButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangList.getRoot());
    });
  });

  describe('on status 503', () => {
    const statusCode: number = 503;
    const getUserProfileInterceptor: string = 'getUserProfileInterceptor';

    beforeEach(() => {
      interceptWithResponse(HttpMethodE2E.GET, '/api/userProfiles/*', {
        statusCode,
        body: { issues: [{ messageCode: MessageCodeE2E.SERVICE_UNAVAILABLE }] },
      }).as(getUserProfileInterceptor);
    });

    it('should manipulate response on vorgang detail navigation', () => {
      vorgangList.getListItem(vorgang.name).getRoot().click();

      waitOfInterceptor(getUserProfileInterceptor).then((interception) => {
        assert(interception.response.statusCode, statusCode.toString());
        exist(vorgangPage.getVorgangDetailHeader().getRoot());
        waitForSpinnerToDisappear();
      });
    });

    it('should show error service unavailable icon', () => {
      exist(
        postfachNachrichtenContainer
          .getListItem(postfachMailItem.subject)
          .getUserProfile()
          .getIconContainer()
          .getErrorServiceUnavailableIcon(),
      );
    });

    it('should hide user profile name', () => {
      notExist(
        postfachNachrichtenContainer
          .getListItem(postfachMailItem.subject)
          .getUserProfile()
          .getName(),
      );
    });

    it('should navigate back', () => {
      vorgangPage.getSubnavigation().getBackButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangList.getRoot());
    });
  });
});
