import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangFormularButtonsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { VorgangListItemE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-item.e2e.component';
import { VorgangE2E, VorgangStatusE2E } from 'apps/alfa-e2e/src/model/vorgang';
import 'cypress-real-events/support';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { contains, enterWith, exist, notExist } from '../../../support/cypress.util';
import {
  initUsermanagerUsers,
  getUserManagerUserSabine,
  getUserSabine,
  loginAsSabine,
} from '../../../support/user-util';
import {
  buildVorgang,
  initSearchIndex,
  initVorgaenge,
  objectIds,
} from '../../../support/vorgang-util';
import { VorgangDetailHeaderE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-detail-header.e2e.component';
import { VorgangAktenzeichenEditE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-aktenzeichen-edit.e2e.component';
import { HeaderE2EComponent } from 'apps/alfa-e2e/src/page-objects/header.po';
import { VorgangSearchE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-search.e2e.component';
import {
  AktenzeichenHistorieItemE2EComponent,
  VorgangFormularDatenHistorieItemE2EComponent,
} from 'apps/alfa-e2e/src/components/vorgang/vorgang.formular-daten.historie.e2e.component';
import { VorgangFormularDatenE2EComponent } from '../../../components/vorgang/vorgang-formular.e2e.component';
import { HistorieHeadlineE2E } from 'apps/alfa-e2e/src/model/historie';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Aktenzeichen anlegen', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const defaultAZ: string = 'default_AZ';
  const noAZ: string = 'kein Aktenzeichen';
  const newAZ: string = 'xyzäöüß!';
  const AZWithBlanks: string = ' aa bb cc   ';
  const trimmedAZ: string = 'aa bb cc';
  const longAZ: string =
    'iiiiiiiiiijjjjjjjjjjkkkkkkkkkkllllllllll_iiiiiiiiiijjjjjjjjjjkkkkkkkkkkllllllllll_iiiiiiiiiijjjjjjjjjjkkkkkkkkkkllllllllll_iiiiiiiiiijjjjjjjjjjkkkkkkkkkkllllllllll_iiiiiiiiiijjjjjjjjjjkkkkkkkkkkllllllllll_iiiiiiiiiijjjjjjjjjjkkkkkkkkkkllllllllll_iiiiiiiiiijjjjjjjjjjkkkkkkkkkkllllllllll_';
  const searchAZ: string = 'AZ Suche';

  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangHeader: VorgangDetailHeaderE2EComponent = vorgangPage.getVorgangDetailHeader();
  const aktenzeichenEditor: VorgangAktenzeichenEditE2EComponent =
    vorgangPage.getAktenzeichenEditor();
  const header: HeaderE2EComponent = mainPage.getHeader();
  const vorgangSearch: VorgangSearchE2EComponent = header.getVorgangSearch();

  const aktenzeichenVorgang: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'Aktenzeichen Vorgang'),
    status: VorgangStatusE2E.NEU,
  };
  const aktenzeichenVorgangEdit: VorgangE2E = {
    ...buildVorgang(objectIds[1], 'Aktenzeichen Vorgang Edit'),
    status: VorgangStatusE2E.ANGENOMMEN,
    aktenzeichen: defaultAZ,
  };
  const aktenzeichenVorgangMissing: VorgangE2E = {
    ...buildVorgang(objectIds[2], 'Missing Aktenzeichen Vorgang'),
    status: VorgangStatusE2E.ANGENOMMEN,
    aktenzeichen: '',
  };
  const aktenzeichenVorgangHistory: VorgangE2E = {
    ...buildVorgang(objectIds[3], 'Aktenzeichen Vorgang Historie'),
    status: VorgangStatusE2E.ANGENOMMEN,
    aktenzeichen: '',
  };
  const aktenzeichenVorgangSearch: VorgangE2E = {
    ...buildVorgang(objectIds[4], 'Aktenzeichen Vorgang Suche'),
    status: VorgangStatusE2E.ANGENOMMEN,
    aktenzeichen: searchAZ,
  };

  const vorgangFormularButtons: VorgangFormularButtonsE2EComponent =
    vorgangPage.getFormularButtons();
  const vorgangDatenFormular: VorgangFormularDatenE2EComponent =
    vorgangPage.getFormularDatenContainer();

  const vorgangShowInSearch: VorgangListItemE2EComponent = vorgangList.getListItem(
    aktenzeichenVorgangSearch.name,
  );

  before(() => {
    initVorgaenge([
      aktenzeichenVorgang,
      aktenzeichenVorgangEdit,
      aktenzeichenVorgangMissing,
      aktenzeichenVorgangHistory,
      aktenzeichenVorgangSearch,
    ]);
    initSearchIndex([aktenzeichenVorgangSearch]);
    initUsermanagerUsers([getUserManagerUserSabine()]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  function setNewAktenzeichen(aktenzeichen: string): void {
    aktenzeichenEditor.getTextEditor().clear().type(aktenzeichen);
    aktenzeichenEditor.getConfirmButton().click();
  }

  describe('check display of edit icon by Vorgang status', () => {
    it('should not show icon on status Neu', () => {
      vorgangList.getListItem(aktenzeichenVorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      notExist(vorgangHeader.getAktenzeichenEdit());
    });

    it('should display icon on status Angenommen', () => {
      vorgangFormularButtons.getAnnehmenButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangHeader.getAktenzeichenEdit());
    });

    it('should show icon on status In Bearbeitung', () => {
      vorgangFormularButtons.getBearbeitenButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangHeader.getAktenzeichenEdit());
    });

    it('should not show icon on all following status', () => {
      vorgangFormularButtons.getBescheidenButton().click();
      waitForSpinnerToDisappear();

      notExist(vorgangHeader.getAktenzeichenEdit());

      vorgangFormularButtons.getAbschliessenButton().click();
      waitForSpinnerToDisappear();

      notExist(vorgangHeader.getAktenzeichenEdit());

      vorgangFormularButtons.getLoeschenAnfordernButton().click();
      waitForSpinnerToDisappear();

      notExist(vorgangHeader.getAktenzeichenEdit());

      vorgangPage.getSubnavigation().getBackButton().click();
    });
  });

  describe('check functionality of Aktenzeichen editing mask', () => {
    it('should display editing mask on clicking edit icon', () => {
      vorgangList.getListItem(aktenzeichenVorgangEdit.name).getRoot().click();
      waitForSpinnerToDisappear();

      vorgangHeader.getAktenzeichenEditButton().click();

      exist(aktenzeichenEditor.getTextEditor());
    });

    it('should paste contents of clipboard', () => {
      const clipboardText: string = 'clipboard';
      cy.window().its('navigator.clipboard').invoke('writeText', clipboardText);
      aktenzeichenEditor.getClipboardButton().click();

      aktenzeichenEditor.getTextEditor().invoke('val').should('equal', clipboardText);
    });

    it('should NOT use new content on cancel', () => {
      aktenzeichenEditor.getCancelButton().click();

      contains(vorgangHeader.getAktenzeichen(), defaultAZ);
    });

    it('should NOT use only blank spaces as Aktenzeichen', () => {
      vorgangHeader.getAktenzeichenEditButton().click();
      setNewAktenzeichen('  ');

      contains(vorgangHeader.getAktenzeichen(), noAZ);
    });

    it('should adopt new valid Aktenzeichen on clicking confirm', () => {
      vorgangHeader.getAktenzeichenEditButton().click();
      setNewAktenzeichen(newAZ);

      contains(vorgangHeader.getAktenzeichen(), newAZ);
    });

    it('should pre-set existing Aktenzeichen on opening mask', () => {
      vorgangHeader.getAktenzeichenEditButton().click();

      aktenzeichenEditor.getTextEditor().invoke('val').should('equal', newAZ);
    });

    it('should trim blank spaces at start and end of Aktenzeichen', () => {
      setNewAktenzeichen(AZWithBlanks);

      contains(vorgangHeader.getAktenzeichen(), trimmedAZ);
    });

    it('should keep edit window open on confirm if Aktenzeichen is too long', () => {
      vorgangHeader.getAktenzeichenEditButton().click();
      cy.window().its('navigator.clipboard').invoke('writeText', longAZ);
      aktenzeichenEditor.getClipboardButton().click();
      aktenzeichenEditor.getConfirmButton().click();

      exist(aktenzeichenEditor.getTextEditor());

      aktenzeichenEditor.getCancelButton().click();
    });

    it('should display placeholder text when Aktenzeichen is missing', () => {
      vorgangPage.getSubnavigation().getBackButton().click();
      vorgangList.getListItem(aktenzeichenVorgangMissing.name).getRoot().click();
      waitForSpinnerToDisappear();
      vorgangHeader.getAktenzeichenEditButton().click();

      aktenzeichenEditor.getTextEditor().invoke('val').should('equal', '');
    });

    it('should not show X button on empty Aktenzeichen', () => {
      notExist(aktenzeichenEditor.getClearButton());
    });

    it('should remove Aktenzeichen on clicking X button', () => {
      aktenzeichenEditor.getTextEditor().clear().type(newAZ);
      aktenzeichenEditor.getClearButton().click();

      aktenzeichenEditor.getTextEditor().invoke('val').should('equal', '');

      aktenzeichenEditor.getCancelButton().click();
      vorgangPage.getSubnavigation().getBackButton().click();
    });
  });

  describe('check Aktenzeichen entries in Historie', () => {
    const userName: string = `${getUserSabine().firstName} ${getUserSabine().lastName}`;

    it('should display history entry on setting new Aktenzeichen', () => {
      vorgangList.getListItem(aktenzeichenVorgangHistory.name).getRoot().click();
      vorgangHeader.getAktenzeichenEditButton().click();
      aktenzeichenEditor.getTextEditor().clear().type(defaultAZ);
      aktenzeichenEditor.getConfirmButton().click();
      vorgangDatenFormular.getHistorieTab().click();

      const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
        .getHistorieContainer()
        .getListItemByIndex(0);

      exist(historieItem.getRoot());
      contains(historieItem.getHeadline(), HistorieHeadlineE2E.AKTENZEICHEN_GEAENDERT);
      contains(historieItem.getUser(), userName);
      exist(historieItem.getExpandButton());

      historieItem.getExpandButton().click();
    });

    it('should show new Aktenzeichen on opening dropdown', () => {
      const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
        .getHistorieContainer()
        .getListItemByIndex(0);
      const aktenzeichenItem: AktenzeichenHistorieItemE2EComponent = historieItem.getAktenzeichen();
      const azText: string = 'Aktenzeichen: ' + defaultAZ;

      contains(aktenzeichenItem.getText(), azText);
    });

    it('should show history entry after changing existing Aktenzeichen', () => {
      vorgangHeader.getAktenzeichenEditButton().click();
      setNewAktenzeichen(newAZ);
      vorgangDatenFormular.getAntragdatenTab().click();
      vorgangDatenFormular.getHistorieTab().click();

      const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
        .getHistorieContainer()
        .getListItemByIndex(1);
      const aktenzeichenItem: AktenzeichenHistorieItemE2EComponent = historieItem.getAktenzeichen();
      const azText: string = 'Aktenzeichen: ' + newAZ;

      historieItem.getExpandButton().click();

      contains(aktenzeichenItem.getText(), azText);
    });

    it('should show history entry after deletion of Aktenzeichen', () => {
      vorgangHeader.getAktenzeichenEditButton().click();
      aktenzeichenEditor.getTextEditor().clear();
      aktenzeichenEditor.getConfirmButton().click();
      vorgangDatenFormular.getAntragdatenTab().click();
      vorgangDatenFormular.getHistorieTab().click();

      const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
        .getHistorieContainer()
        .getListItemByIndex(2);

      contains(historieItem.getHeadline(), HistorieHeadlineE2E.AKTENZEICHEN_GELOESCHT);

      vorgangPage.getSubnavigation().getBackButton().click();
    });
  });

  describe('search for Aktenzeichen', () => {
    it('should find Vorgang with existing Aktenzeichen', () => {
      vorgangList.getListItem(aktenzeichenVorgangSearch.name).getRoot().click();
      enterWith(vorgangSearch.getInput(), searchAZ);
      //currently the first search only works if you manually send an Enter after a short delay
      cy.wait(1000);
      vorgangSearch.getInput().type('{enter}');

      exist(vorgangShowInSearch.getRoot());
    });

    it('should not find the same Vorgang after changing its Aktenzeichen', () => {
      vorgangList.getListItem(aktenzeichenVorgangSearch.name).getRoot().click();
      vorgangHeader.getAktenzeichenEditButton().click();
      setNewAktenzeichen(newAZ);
      enterWith(vorgangSearch.getInput(), searchAZ);

      notExist(vorgangShowInSearch.getRoot());
    });
  });
});
