/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { faker } from '@faker-js/faker';
import { SnackBarE2EComponent } from 'apps/alfa-e2e/src/components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { WiedervorlageInVorgangE2EComponent } from '../../../components/wiedervorlage/wiedervorlage-in-vorgang.e2e.component';
import { WiedervorlageE2EComponent } from '../../../components/wiedervorlage/wiedervorlage-page.e2e.component';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { WiedervorlagePage } from '../../../page-objects/wiedervorlage.po';
import { dropCollections, wait } from '../../../support/cypress-helper';
import { exist, haveText, haveValue, notExist } from '../../../support/cypress.util';
import { MessagesE2E } from '../../../support/messages';
import { formatDateLocal } from '../../../support/tech.util';
import { loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgaenge } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe.skip('Wiedervorlage', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
  const snackbar: SnackBarE2EComponent = mainPage.getSnackBar();

  const vorgangPage: VorgangPage = new VorgangPage();
  const wiedervorlagePage: WiedervorlagePage = new WiedervorlagePage();
  const wiedervorlageContainer: WiedervorlageE2EComponent =
    wiedervorlagePage.getWiedervorlageContainer();

  const vorgang: VorgangE2E = createVorgang();

  const wiedervorlageBetreff: string = 'TestWiedervorlage';
  const wiedervorlageBeschreibung: string = 'Beschreibung einer Wiedervorlage';
  const wiedervorlageFrist: string = formatDateLocal(new Date(), 'dd.MM.yyyy');

  before(() => {
    initVorgaenge([vorgang]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('Open Vorgang-Detail-Page', () => {
    it('should show vorgang detail by click on vorgang in list', () => {
      vorgangList.getListItem(vorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });
  });

  describe('create wiedervorlage', () => {
    it('should open wiedervorlage pag by click on create wiedervorlage button', () => {
      vorgangPage.getWiedervorlagenContainer().getCreateWiedervorlageButton().click();
      waitForSpinnerToDisappear();

      exist(wiedervorlagePage.getSubnavigation().getRoot());
    });

    it('should show validation error if betreff is to short', () => {
      wiedervorlageContainer.getBetreff().clear().type('Ab');

      wiedervorlageContainer.getSpeichernButton().click();
      waitForSpinnerToDisappear();

      exist(wiedervorlageContainer.getBetreffError());
    });

    it('should show validation error if betreff is to long', () => {
      wiedervorlageContainer.getBetreff().clear().type(faker.random.alphaNumeric(41));

      wiedervorlageContainer.getSpeichernButton().click();
      waitForSpinnerToDisappear();

      exist(wiedervorlageContainer.getBetreffError());
    });

    it('should show validation error on invalid frist', () => {
      wiedervorlageContainer.getBeschreibung().clear().type('Dies ist ein Test');
      wiedervorlageContainer.getFrist().clear().type('1.1.2020');

      wiedervorlageContainer.getSpeichernButton().click();
      waitForSpinnerToDisappear();

      exist(wiedervorlageContainer.getDatumError());
    });

    it('should show snackbar on valid data', () => {
      wiedervorlageContainer.getBetreff().clear().type(wiedervorlageBetreff);
      wiedervorlageContainer.getBeschreibung().clear().type(wiedervorlageBeschreibung);
      wiedervorlageContainer.getFrist().clear().type(wiedervorlageFrist);

      wiedervorlageContainer.getSpeichernButton().click();
      waitForSpinnerToDisappear();

      exist(snackbar.getMessage());
      haveText(
        snackbar.getMessage(),
        MessagesE2E.WIEDERVORLAGE_ANGELEGT.replace('%s', wiedervorlageBetreff),
      );
    });

    it('should close snackbar on click on close button', () => {
      snackbar.getCloseButton().click();

      notExist(snackbar.getMessage());
    });

    it('should show wiedervorlage on vorgang page', () => {
      const wiedervorlage = vorgangPage
        .getWiedervorlagenContainer()
        .getWiedervorlage(wiedervorlageBetreff);

      exist(wiedervorlage.getRoot());

      haveText(wiedervorlage.getFrist(), formatDateLocal(new Date(), 'dd. MMM'));
      haveText(wiedervorlage.getBetreff(), wiedervorlageBetreff);
    });

    it('should show vorganglist on navigate back', () => {
      vorgangPage.getSubnavigation().getBackButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangList.getRoot());
    });
  });

  describe('edit wiedervorlage', () => {
    it('should open Vorgang-Detail-Page on click on vorgang in list', () => {
      vorgangList.getListItem(vorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should open Wiedervorlage-Page by click on wiedervorlage ', () => {
      const wiedervorlageInVorgang: WiedervorlageInVorgangE2EComponent = vorgangPage
        .getWiedervorlagenContainer()
        .getWiedervorlage(wiedervorlageBetreff);

      wait(500);
      wiedervorlageInVorgang.getLink().click();
      waitForSpinnerToDisappear();

      exist(wiedervorlagePage.getSubnavigation().getRoot());
    });

    it('should have input values', () => {
      haveValue(wiedervorlageContainer.getBeschreibung(), wiedervorlageBeschreibung);
      haveValue(wiedervorlageContainer.getBetreff(), wiedervorlageBetreff);
      haveValue(wiedervorlageContainer.getFrist(), wiedervorlageFrist);
    });

    it('should show snackbar on valid data', () => {
      wiedervorlageContainer.getBetreff().clear().type('Editierter Betreff');
      wiedervorlageContainer.getBeschreibung().clear().type('Diese Wiedervorlage wurde bearbeitet');
      wiedervorlageContainer.getFrist().clear().type(formatDateLocal(new Date(), 'dd.MM.yyyy'));

      wiedervorlageContainer.getSpeichernButton().click();
      waitForSpinnerToDisappear();

      exist(snackbar.getMessage());
      haveText(
        snackbar.getMessage(),
        MessagesE2E.WIEDERVORLAGE_GESPEICHERT.replace('%s', 'Editierter Betreff'),
      );
    });

    it('should close snackbar on click on close button', () => {
      snackbar.getCloseButton().click();

      notExist(snackbar.getMessage());
    });

    it('should navigate after click on "Speichern"', () => {
      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should show existing wiedervorlage', () => {
      const wiedervorlage = vorgangPage
        .getWiedervorlagenContainer()
        .getWiedervorlage('Editierter Betreff');

      exist(wiedervorlage.getRoot());

      haveText(wiedervorlage.getFrist(), formatDateLocal(new Date(), 'dd. MMM'));
      haveText(wiedervorlage.getBetreff(), 'Editierter Betreff');
    });

    it('should not show wiedervorlage with legacy data', () => {
      const wiedervorlageInVorgang: WiedervorlageInVorgangE2EComponent = vorgangPage
        .getWiedervorlagenContainer()
        .getWiedervorlage(wiedervorlageBetreff);

      notExist(wiedervorlageInVorgang.getRoot());
    });
  });

  describe('remove beschreibung from wiedervorlage', () => {
    it('should open Wiedervorlage-Page by click on wiedervorlage ', () => {
      const wiedervorlageInVorgang: WiedervorlageInVorgangE2EComponent = vorgangPage
        .getWiedervorlagenContainer()
        .getWiedervorlage('Editierter Betreff');

      wiedervorlageInVorgang.getLink().click();
      waitForSpinnerToDisappear();

      exist(wiedervorlagePage.getSubnavigation().getRoot());
    });

    it('should have beschreibung', () => {
      haveValue(wiedervorlageContainer.getBeschreibung(), 'Diese Wiedervorlage wurde bearbeitet');
    });

    it('should show snackbar after clear beschreibung', () => {
      wiedervorlageContainer.getBeschreibung().clear();

      wiedervorlageContainer.getSpeichernButton().click();
      waitForSpinnerToDisappear();

      exist(snackbar.getMessage());
      haveText(
        snackbar.getMessage(),
        MessagesE2E.WIEDERVORLAGE_GESPEICHERT.replace('%s', 'Editierter Betreff'),
      );
    });

    it('should close snackbar on click on close button', () => {
      snackbar.getCloseButton().click();

      notExist(snackbar.getMessage());
    });

    it('should navigate after click on "Speichern"', () => {
      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should navigate to Wiedervorlage formular', () => {
      const wiedervorlageInVorgang: WiedervorlageInVorgangE2EComponent = vorgangPage
        .getWiedervorlagenContainer()
        .getWiedervorlage('Editierter Betreff');

      wiedervorlageInVorgang.getLink().click();
      waitForSpinnerToDisappear();

      exist(wiedervorlagePage.getSubnavigation().getRoot());
    });

    it('should show empty beschreibung', () => {
      haveText(wiedervorlageContainer.getBeschreibung(), '');
    });
  });
});
