/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { FileDataE2E } from './binary-file';
import { KommentarE2E } from './kommentar';
import { DateE2E, ObjectIdE2E, ResourceUriE2E } from './util';
import { WiedervorlageE2E } from './wiedervorlage';

export enum ClientAttributeNameE2E {
  NEXT_WIEDERVORLAGE_FRIST = 'nextWiedervorlageFrist',
  HAS_POSTFACH_NACHRICHT = 'hasPostfachNachricht',
  HAS_NEW_POSTFACH_NACHRICHT = 'hasNewPostfachNachricht',
}

export enum AccessPermissionE2E {
  READ_ONLY = 'READ_ONLY',
  READ_WRITE = 'READ_WRITE',
}

export interface ClientAttributeE2E {
  access: AccessPermissionE2E;
  value: string | number | boolean;
  _class: string;
}

export interface ClientAttributesE2E {
  [client: string]: {
    [attributeName: string]: ClientAttributeE2E;
  };
}

export class VorgangE2E {
  _id: ObjectIdE2E;
  version: string;
  createdAt: DateE2E;
  name: string;
  status: VorgangStatusE2E;
  aktenzeichen: string;
  nummer: string;
  header: VorgangHeaderE2E;
  eingangs: EingangE2E[];
  wiedervorlages: WiedervorlageE2E[];
  kommentars: KommentarE2E[];
  assignedTo: ResourceUriE2E;
  clientAttributes: ClientAttributesE2E;
}

export class VorgangHeaderE2E {
  serviceKonto: ServiceKontoE2E;
}

export class ServiceKontoE2E {
  type: string;
  postfachAddress: PostfachAddressE2E[];
}

export class PostfachAddressE2E {
  type: number;
  identifier: ReadonlyMap<string, object>;
}

export class EingangE2E {
  header: EingangHeaderE2E;
  antragsteller?: AntragstellerE2E;
  zustaendigeStelle?: ZustaendigeStelleE2E;
  attachments: AttachmentE2E[];
  representations: FileDataE2E[];
  numberOfAttachments: number;
  numberOfRepresentations: number;
  formData;
}

export class EingangHeaderE2E {
  requestId: string;
  createdAt: DateE2E;
  formId: string;
  formEngineName: string;
}

export class AntragstellerE2E {
  anrede: string;
  nachname: string;
  vorname: string;
  geburtsort: string;
  geburtsname: string;
  postfachId: string;
  geburtsdatum: string;
  email: string;
  telefon: string;
  strasse: string;
  hausnummer: string;
  plz: string;
  ort: string;
}

export class ZustaendigeStelleE2E {
  organisationseinheitenId: string;
}

export class AttachmentE2E {
  name: string;
  files: FileDataE2E[];
}

export enum VorgangStatusE2E {
  NEU = 'NEU',
  ANGENOMMEN = 'ANGENOMMEN',
  VERWORFEN = 'VERWORFEN',
  IN_BEARBEITUNG = 'IN_BEARBEITUNG',
  BESCHIEDEN = 'BESCHIEDEN',
  ABGESCHLOSSEN = 'ABGESCHLOSSEN',
  WEITERGELEITET = 'WEITERGELEITET',
  ZU_LOESCHEN = 'ZU_LOESCHEN',
}

export const vorgangStatusLabelE2E = {
  [VorgangStatusE2E.NEU]: 'Neu',
  [VorgangStatusE2E.ANGENOMMEN]: 'Angenommen',
  [VorgangStatusE2E.VERWORFEN]: 'Verworfen',
  [VorgangStatusE2E.IN_BEARBEITUNG]: 'In Bearbeitung',
  [VorgangStatusE2E.BESCHIEDEN]: 'Beschieden',
  [VorgangStatusE2E.ABGESCHLOSSEN]: 'Abgeschlossen',
  [VorgangStatusE2E.WEITERGELEITET]: 'Weitergeleitet',
  [VorgangStatusE2E.ZU_LOESCHEN]: 'Zu Löschen',
};

export enum VorgangMessagesE2E {
  ANGENOMMEN = 'Der Vorgang wurde angenommen.',
  VERWORFEN = 'Der Vorgang wurde verworfen.',
  ZURUECKGEHOLT = 'Der Vorgang wurde zurückgeholt.',
  BEARBEITET = 'Der Vorgang wurde in die Bearbeitung übernommen.',
  BESCHIEDEN = 'Der Vorgang wurde beschieden.',
  ZURUECKGESTELLT = 'Der Vorgang wurde zurückgestellt.',
  ABGESCHLOSSEN = 'Der Vorgang wurde abgeschlossen.',
  WIEDEREROEFFNET = 'Der Vorgang wurde wiedereröffnet.',
  LOESCHEN_ANFORDERN = 'Für den Vorgang wurde eine Löschanforderung gestellt.',
  ENDGUELTIG_LOESCHEN = 'Der Vorgang wurde gelöscht.',
  WIEDERVORLAGE_BEARBEITEN_NICHT_MOEGLICH = 'Im Status "Zu löschen" ist die Bearbeitung von Wiedervorlagen nicht möglich.',
}

export const NO_AKTENZEICHEN: string = 'kein Aktenzeichen zugewiesen';
