/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { faker } from '@faker-js/faker';
import { BinaryFileListLinkRel } from 'libs/binary-file-shared/src/lib/binary-file.linkrel';
import {
  BinaryFile,
  BinaryFileListResource,
  BinaryFileResource,
} from 'libs/binary-file-shared/src/lib/binary-file.model';
import { toResource } from 'libs/tech-shared/test/resource';
import { times } from 'lodash-es';

export function createBinaryFile(): BinaryFile {
  return {
    name: faker.lorem.slug(10),
    size: faker.datatype.number({ min: 1024, max: 10000000000 }),
    contentType: faker.lorem.word(),
  };
}

export function createBinaryFileResource(linkRelations: string[] = []): BinaryFileResource {
  return toResource(createBinaryFile(), linkRelations);
}

export function createBinaryFileResources(linkRelations: string[] = []): BinaryFileResource[] {
  return times(10, () => toResource(createBinaryFileResource(), [...linkRelations]));
}

export function createBinaryFileListResource(
  resources?: BinaryFileResource[],
  linkRelations: string[] = [],
): BinaryFileListResource {
  return toResource({}, [...linkRelations], {
    [BinaryFileListLinkRel.FILE_LIST]: resources ? resources : createBinaryFileResources(),
  });
}
