/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { CommandOrder, CommandOrderType } from '@alfa-client/command-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { createCommandResource } from 'libs/command-shared/test/command';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { HistorieItemAssignUserContainerComponent } from '../historie-item-assign-user-container/historie-item-assign-user-container.component';
import { HistorieItemForwardingComponent } from '../historie-item-forwarding/historie-item-forwarding.component';
import { HistorieItemHeaderComponent } from '../historie-item-header/historie-item-header.component';
import { HistorieItemKommentarComponent } from '../historie-item-kommentar/historie-item-kommentar.component';
import { HistorieItemPostfachNachrichtComponent } from '../historie-item-postfach-nachricht/historie-item-postfach-nachricht.component';
import { HistorieItemUnknownComponent } from '../historie-item-unknown/historie-item-unknown.component';
import { HistorieItemVorgangStatusComponent } from '../historie-item-vorgang-status/historie-item-vorgang-status.component';
import { HistorieItemWiedervorlageComponent } from '../historie-item-wiedervorlage/historie-item-wiedervorlage.component';
import { HistorieListItemComponent } from './historie-list-item.component';

describe('HistorieListItemComponent', () => {
  let component: HistorieListItemComponent;
  let fixture: ComponentFixture<HistorieListItemComponent>;

  const assignUserItem: string = getDataTestIdOf('historie-item-assign-user');
  const kommentarItem: string = getDataTestIdOf('historie-item-kommentar');
  const forwardingItem: string = getDataTestIdOf('historie-item-forwarding');
  const postfachNachrichtItem: string = getDataTestIdOf('historie-item-postfach-nachricht');
  const vorgangItem: string = getDataTestIdOf('historie-item-vorgang');
  const wiedervorlageItem: string = getDataTestIdOf('historie-item-wiedervorlage');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        HistorieListItemComponent,
        MockComponent(HistorieItemHeaderComponent),
        MockComponent(HistorieItemWiedervorlageComponent),
        MockComponent(HistorieItemKommentarComponent),
        MockComponent(HistorieItemVorgangStatusComponent),
        MockComponent(HistorieItemUnknownComponent),
        MockComponent(HistorieItemPostfachNachrichtComponent),
        MockComponent(HistorieItemForwardingComponent),
        MockComponent(HistorieItemAssignUserContainerComponent),
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(HistorieListItemComponent);
    component = fixture.componentInstance;
    component.historieCommand = createCommandResource();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('decision of item component', () => {
    describe('on ' + CommandOrderType.USER + ' type ', () => {
      const order: CommandOrder[] = [CommandOrder.ASSIGN_USER];

      it.each(order)('should show item on order s%', (order: CommandOrder) => {
        component.historieCommand = { ...createCommandResource(), order };
        fixture.detectChanges();

        const itemComponent = getElementFromFixture(fixture, assignUserItem);

        expect(itemComponent).toBeInstanceOf(HTMLElement);
      });
    });

    describe('on ' + CommandOrderType.KOMMENTAR + ' type ', () => {
      const order: CommandOrder[] = [CommandOrder.CREATE_KOMMENTAR, CommandOrder.EDIT_KOMMENTAR];

      it.each(order)('should show item on order %s', (order: CommandOrder) => {
        component.historieCommand = { ...createCommandResource(), order };
        fixture.detectChanges();

        const itemComponent = getElementFromFixture(fixture, kommentarItem);

        expect(itemComponent).toBeInstanceOf(HTMLElement);
      });
    });

    describe('on ' + CommandOrderType.FORWARDING + ' type ', () => {
      const order: CommandOrder[] = [
        CommandOrder.REDIRECT_VORGANG,
        CommandOrder.FORWARD_SUCCESSFULL,
        CommandOrder.FORWARD_FAILED,
      ];

      it.each(order)('should show item on order %s', (order: CommandOrder) => {
        component.historieCommand = { ...createCommandResource(), order };
        fixture.detectChanges();

        const itemComponent = getElementFromFixture(fixture, forwardingItem);

        expect(itemComponent).toBeInstanceOf(HTMLElement);
      });
    });

    describe('on ' + CommandOrderType.POSTFACH_NACHRICHT + ' type ', () => {
      const order: CommandOrder[] = [
        CommandOrder.SEND_POSTFACH_NACHRICHT,
        CommandOrder.RECEIVE_POSTFACH_NACHRICHT,
      ];

      it.each(order)('should show item on order %s', (order: CommandOrder) => {
        component.historieCommand = { ...createCommandResource(), order };
        fixture.detectChanges();

        const itemComponent = getElementFromFixture(fixture, postfachNachrichtItem);

        expect(itemComponent).toBeInstanceOf(HTMLElement);
      });
    });

    describe('on ' + CommandOrderType.VORGANG + ' type ', () => {
      const order: CommandOrder[] = [
        CommandOrder.VORGANG_ANNEHMEN,
        CommandOrder.VORGANG_VERWERFEN,
        CommandOrder.VORGANG_ZURUECKHOLEN,
        CommandOrder.VORGANG_BEARBEITEN,
        CommandOrder.VORGANG_BESCHEIDEN,
        CommandOrder.VORGANG_ZURUECKSTELLEN,
        CommandOrder.VORGANG_ABSCHLIESSEN,
        CommandOrder.VORGANG_WIEDEREROEFFNEN,
      ];

      it.each(order)('should show item on order %s', (order: CommandOrder) => {
        component.historieCommand = { ...createCommandResource(), order };
        fixture.detectChanges();

        const itemComponent = getElementFromFixture(fixture, vorgangItem);

        expect(itemComponent).toBeInstanceOf(HTMLElement);
      });
    });

    describe('on ' + CommandOrderType.WIEDERVORLAGE + ' type ', () => {
      const order: CommandOrder[] = [
        CommandOrder.CREATE_WIEDERVORLAGE,
        CommandOrder.EDIT_WIEDERVORLAGE,
        CommandOrder.WIEDERVORLAGE_ERLEDIGEN,
        CommandOrder.WIEDERVORLAGE_WIEDEREROEFFNEN,
      ];

      it.each(order)('should show item on order %s', (order: CommandOrder) => {
        component.historieCommand = { ...createCommandResource(), order };
        fixture.detectChanges();

        const itemComponent = getElementFromFixture(fixture, wiedervorlageItem);

        expect(itemComponent).toBeInstanceOf(HTMLElement);
      });
    });
  });
});
