/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, Inject, OnInit } from '@angular/core';
import { MAT_DIALOG_DATA } from '@angular/material/dialog';
import { CommandResource } from '@alfa-client/command-shared';
import { StateResource, createEmptyStateResource, isNotNil } from '@alfa-client/tech-shared';
import { DialogService } from '@alfa-client/ui';
import { PostfachMailFormDialogData } from 'libs/postfach-shared/src/lib/postfach.model';
import { Observable, of, tap } from 'rxjs';
import { PostfachMailFormservice } from './postfach-mail.formservice';

import * as CommandUtil from '../../../../command-shared/src/lib/command.util';

@Component({
  selector: 'alfa-postfach-mail-form',
  templateUrl: './postfach-mail-form.component.html',
  styleUrls: ['./postfach-mail-form.component.scss'],
  providers: [PostfachMailFormservice],
})
export class PostfachMailFormComponent implements OnInit {
  public readonly formServiceClass = PostfachMailFormservice;

  public sendInProgress$: Observable<StateResource<CommandResource>> = of(
    createEmptyStateResource<CommandResource>(),
  );

  constructor(
    private dialogService: DialogService,
    @Inject(MAT_DIALOG_DATA) public dialogData: PostfachMailFormDialogData,
    public formService: PostfachMailFormservice,
  ) {}

  ngOnInit(): void {
    if (isNotNil(this.dialogData.postfachNachricht)) {
      this.patchPostfachNachricht();
    }
  }

  patchPostfachNachricht(): void {
    this.formService.patch(this.dialogData.postfachNachricht);
  }

  public submit(): void {
    this.sendInProgress$ = this.formService
      .submit()
      .pipe(
        tap((commandStateResource: StateResource<CommandResource>) =>
          this.closeDialogsOnSuccessfulSubmit(commandStateResource),
        ),
      );
  }

  closeDialogsOnSuccessfulSubmit(commandStateResource: StateResource<CommandResource>): void {
    if (CommandUtil.isSuccessfulDone(commandStateResource.resource)) {
      this.dialogService.closeAll();
    }
  }
}
