/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HttpErrorResponse } from '@angular/common/http';
import { faker } from '@faker-js/faker';
import { ApiError, Issue, IssueParam } from '../src/lib/tech.model';

export function createIssueParam(): IssueParam {
  return {
    name: faker.random.word(),
    value: faker.random.word(),
  };
}

export function createIssue(): Issue {
  return {
    field: faker.random.word(),
    message: faker.random.words(10),
    messageCode: faker.random.words(5),
    parameters: [createIssueParam()],
  };
}

export function createApiError(): ApiError {
  return {
    issues: [createIssue()],
  };
}

export function createHttpErrorResponse(apiError: ApiError = null): HttpErrorResponse {
  return <HttpErrorResponse>{
    error: {
      error: apiError ?? createApiError(),
    },
  };
}
