/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule } from '@angular/forms';
import { MatIcon } from '@angular/material/icon';
import { MatFormFieldModule } from '@angular/material/form-field';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { ConvertForDataTestPipe } from '@alfa-client/tech-shared';
import { IconButtonWithSpinnerComponent, UiModule } from '@alfa-client/ui';
import { MockComponent } from 'ng-mocks';
import { EnumEditorComponent } from './enum-editor.component';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { MatCheckboxChange } from '@angular/material/checkbox';

describe('EnumEditorComponent', () => {
  let component: EnumEditorComponent;
  let fixture: ComponentFixture<EnumEditorComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        EnumEditorComponent,
        MatIcon,
        ConvertForDataTestPipe,
        MockComponent(IconButtonWithSpinnerComponent),
      ],
      imports: [UiModule, MatFormFieldModule, ReactiveFormsModule, BrowserAnimationsModule],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(EnumEditorComponent);
    component = fixture.componentInstance;
    component.defaultItem = { type: 'enumType', icon: 'x', text: 'y' };
    component.checkedItem = { type: 'positive', icon: 'x', text: 'y' };
    component.uncheckedItem = { type: 'negative', icon: 'x', text: 'y' }
    component.itemList = [{ type: 'enumType', icon: 'x', text: 'y' }];
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('setSelection', () => {
    const writeMethodMock = jest.fn();

    beforeEach(() => {
      component.writeValue = writeMethodMock;
    })

    it('should write checked value on selected', () => {
      component.setSelection({ source: null, checked: true });

      expect(writeMethodMock).toHaveBeenCalledWith('positive');
    })

    it('should write unchecked value on selected', () => {
      component.setSelection({ source: null, checked: false });

      expect(writeMethodMock).toHaveBeenCalledWith('negative');
    })
  })
});
