/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ApiRootLinkRel } from '@alfa-client/api-root-shared';
import { HasLinkPipe, createStateResource } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { OzgcloudIconComponent, PostfachIconComponent } from '@alfa-client/ui';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { HasAnyLinkPipe } from 'libs/tech-shared/src/lib/pipe/has-any-link.pipe';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangStatistic } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { VorgangSearchViewItemContainerComponent } from './vorgang-search-view-item-container/vorgang-search-view-item-container.component';
import { VorgangViewItemContainerComponent } from './vorgang-view-item-container/vorgang-view-item-container.component';
import { VorgangViewItemComponent } from './vorgang-view-item-container/vorgang-view-item/vorgang-view-item.component';
import { VorgangViewsMenuComponent } from './vorgang-views-menu.component';
import { WiedervorlageIconComponent } from '../../../../../../vorgang-shared-ui/src/lib/wiedervorlage-icon/wiedervorlage-icon.component';

describe('VorgangViewsMenuComponent', () => {
  let component: VorgangViewsMenuComponent;
  let fixture: ComponentFixture<VorgangViewsMenuComponent>;

  const neuView: string = getDataTestIdOf('vorgang-neu-view');
  const angenommenView: string = getDataTestIdOf('vorgang-angenommen-view');
  const inBearbeitungView: string = getDataTestIdOf('vorgang-in-bearbeitung-view');
  const beschiedenView: string = getDataTestIdOf('vorgang-beschieden-view');
  const abgeschlossenView: string = getDataTestIdOf('vorgang-abgeschlossen-view');
  const verworfenView: string = getDataTestIdOf('vorgang-verworfen-view');
  const wiedervorlagenView: string = getDataTestIdOf('vorgang-wiedervorlagen-view');
  const ungeleseneNachrichtenView: string = getDataTestIdOf('vorgang-ungelesene-nachrichten-view');
  const vorgangListView: string = getDataTestIdOf('vorgang-vorgang-list-view');
  const searchView: string = getDataTestIdOf('vorgang-search-view');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        WiedervorlageIconComponent,
        VorgangViewsMenuComponent,
        HasLinkPipe,
        HasAnyLinkPipe,
        MockComponent(VorgangViewItemContainerComponent),
        MockComponent(VorgangViewItemComponent),
        MockComponent(VorgangSearchViewItemContainerComponent),
        MockComponent(OzgcloudIconComponent),
        MockComponent(PostfachIconComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangViewsMenuComponent);
    component = fixture.componentInstance;
    component.vorgangStatisticStateResource = createStateResource(createVorgangStatistic());
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('neu view', () => {
    it.each([
      ApiRootLinkRel.ALLE_VORGAENGE_NEU,
      ApiRootLinkRel.MEINE_VORGAENGE_NEU,
      ApiRootLinkRel.UNASSIGNED_NEU,
    ])('should show if %s link exists', (linkRel: string) => {
      component.apiRootResource = createApiRootResource([linkRel]);
      fixture.detectChanges();

      existsAsHtmlElement(fixture, neuView);
    });

    it('should hide if both links not exists', () => {
      component.apiRootResource = createApiRootResource();
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, neuView);
    });
  });

  describe('angenommen view', () => {
    it.each([
      ApiRootLinkRel.ALLE_VORGAENGE_ANGENOMMEN,
      ApiRootLinkRel.MEINE_VORGAENGE_ANGENOMMEN,
      ApiRootLinkRel.UNASSIGNED_ANGENOMMEN,
    ])('should show if %s link exists', (linkRel: string) => {
      component.apiRootResource = createApiRootResource([linkRel]);
      fixture.detectChanges();

      existsAsHtmlElement(fixture, angenommenView);
    });

    it('should hide if both links not exists', () => {
      component.apiRootResource = createApiRootResource();
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, angenommenView);
    });
  });

  describe('in bearbeitung view', () => {
    it.each([
      ApiRootLinkRel.ALLE_VORGAENGE_IN_BEARBEITUNG,
      ApiRootLinkRel.MEINE_VORGAENGE_IN_BEARBEITUNG,
      ApiRootLinkRel.UNASSIGNED_IN_BEARBEITUNG,
    ])('should show if %s link exists', (linkRel: string) => {
      component.apiRootResource = createApiRootResource([linkRel]);
      fixture.detectChanges();

      existsAsHtmlElement(fixture, inBearbeitungView);
    });

    it('should hide if both links not exists', () => {
      component.apiRootResource = createApiRootResource();
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, inBearbeitungView);
    });
  });

  describe('beschieden view', () => {
    it.each([
      ApiRootLinkRel.ALLE_VORGAENGE_BESCHIEDEN,
      ApiRootLinkRel.MEINE_VORGAENGE_BESCHIEDEN,
      ApiRootLinkRel.UNASSIGNED_BESCHIEDEN,
    ])('should show if %s link exists', (linkRel: string) => {
      component.apiRootResource = createApiRootResource([linkRel]);
      fixture.detectChanges();

      existsAsHtmlElement(fixture, beschiedenView);
    });

    it('should hide if both links not exists', () => {
      component.apiRootResource = createApiRootResource();
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, beschiedenView);
    });
  });

  describe('abgeschlossen view', () => {
    it.each([
      ApiRootLinkRel.ALLE_VORGAENGE_ABGESCHLOSSEN,
      ApiRootLinkRel.MEINE_VORGAENGE_ABGESCHLOSSEN,
      ApiRootLinkRel.UNASSIGNED_ABGESCHLOSSEN,
    ])('should show if %s link exists', (linkRel: string) => {
      component.apiRootResource = createApiRootResource([linkRel]);
      fixture.detectChanges();

      existsAsHtmlElement(fixture, abgeschlossenView);
    });

    it('should hide if both links not exists', () => {
      component.apiRootResource = createApiRootResource();
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, beschiedenView);
    });
  });

  describe('verworfen view', () => {
    it.each([
      ApiRootLinkRel.ALLE_VORGAENGE_VERWORFEN,
      ApiRootLinkRel.MEINE_VORGAENGE_VERWORFEN,
      ApiRootLinkRel.UNASSIGNED_VERWORFEN,
    ])('should show if %s link exists', (linkRel: string) => {
      component.apiRootResource = createApiRootResource([linkRel]);
      fixture.detectChanges();

      existsAsHtmlElement(fixture, verworfenView);
    });

    it('should hide if both links not exists', () => {
      component.apiRootResource = createApiRootResource();
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, verworfenView);
    });
  });

  describe('wiedervorlagen view', () => {
    it.each([
      ApiRootLinkRel.ALL_WIEDERVORLAGEN,
      ApiRootLinkRel.MY_WIEDERVORLAGEN,
      ApiRootLinkRel.UNASSIGNED_WIEDERVORLAGEN,
    ])('should show if %s link exists', (linkRel: string) => {
      component.apiRootResource = createApiRootResource([linkRel]);
      fixture.detectChanges();

      existsAsHtmlElement(fixture, wiedervorlagenView);
    });

    it('should hide if link not exists', () => {
      component.apiRootResource = createApiRootResource();
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, wiedervorlagenView);
    });
  });

  describe('ungelesene nachrichten view', () => {
    it.each([
      ApiRootLinkRel.ALL_UNGELESENE_NACHRICHTEN,
      ApiRootLinkRel.MY_UNGELESENE_NACHRICHTEN,
      ApiRootLinkRel.UNASSIGNED_UNGELESENE_NACHRICHTEN,
    ])('should show if %s link exists', (linkRel: string) => {
      component.apiRootResource = createApiRootResource([linkRel]);
      fixture.detectChanges();

      existsAsHtmlElement(fixture, ungeleseneNachrichtenView);
    });

    it('should hide if link not exists', () => {
      component.apiRootResource = createApiRootResource();
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, ungeleseneNachrichtenView);
    });
  });

  describe('vorgangList view', () => {
    it.each([
      ApiRootLinkRel.ALLE_VORGAENGE,
      ApiRootLinkRel.MEINE_VORGAENGE,
      ApiRootLinkRel.UNASSIGNED,
    ])('should show if %s link exists', (linkRel: string) => {
      component.apiRootResource = createApiRootResource([linkRel]);
      fixture.detectChanges();

      existsAsHtmlElement(fixture, vorgangListView);
    });

    it('should hide if link not exists', () => {
      component.apiRootResource = createApiRootResource();
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, vorgangListView);
    });
  });

  describe('search view', () => {
    it.each([
      ApiRootLinkRel.SEARCH_ALLE,
      ApiRootLinkRel.SEARCH_MEINE,
      ApiRootLinkRel.SEARCH_UNASSIGNED,
    ])('should show if %s link exists', (linkRel: string) => {
      component.apiRootResource = createApiRootResource([linkRel]);
      fixture.detectChanges();

      existsAsHtmlElement(fixture, searchView);
    });

    it('should hide if no link not exists', () => {
      component.apiRootResource = createApiRootResource();
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, searchView);
    });
  });
});
