/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { ToTrafficLightPipe } from '@alfa-client/tech-shared';
import { MatTooltipDirective } from '@alfa-client/ui';
import { createWiedervorlageResource } from 'libs/wiedervorlage-shared/test/wiedervorlage';
import { MockDirective } from 'ng-mocks';
import { WiedervorlageStatusComponent } from './wiedervorlage-status.component';

const doneIcon: string = '[data-test-class="done-icon"]';

describe('WiedervorlageStatusComponent', () => {
  let component: WiedervorlageStatusComponent;
  let fixture: ComponentFixture<WiedervorlageStatusComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        WiedervorlageStatusComponent,
        ToTrafficLightPipe,
        MockDirective(MatTooltipDirective),
        MatIcon,
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(WiedervorlageStatusComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('status dot', () => {
    it('should have no icon initially', () => {
      const doneIconElement = fixture.nativeElement.querySelector(doneIcon);

      expect(doneIconElement).not.toBeInstanceOf(HTMLElement);
    });

    it('should have no icon if wiedervorlage is not done', () => {
      component.wiedervorlageResource = { ...createWiedervorlageResource(), done: false };
      fixture.detectChanges();

      const doneIconElement = fixture.nativeElement.querySelector(doneIcon);

      expect(doneIconElement).not.toBeInstanceOf(HTMLElement);
    });

    it('should have icon if wiedervorlage is done', () => {
      component.wiedervorlageResource = { ...createWiedervorlageResource(), done: true };
      fixture.detectChanges();

      const doneIconElement = fixture.nativeElement.querySelector(doneIcon);

      expect(doneIconElement).toBeInstanceOf(HTMLElement);
    });
  });
});
