package de.ozgcloud.alfa.common;

import java.time.ZonedDateTime;

import de.ozgcloud.alfa.common.file.OzgFile;
import de.xoev.xdomea.PrimaerdokumentType;

public class PrimaerdokumentTypeBuilder {

	private OzgFile ozgFile;
	private String ersteller;
	private ZonedDateTime createdAt;

	public static PrimaerdokumentTypeBuilder builder() {
		return new PrimaerdokumentTypeBuilder();
	}

	public PrimaerdokumentTypeBuilder withOzgFile(OzgFile ozgFile) {
		this.ozgFile = ozgFile;
		return this;
	}

	public PrimaerdokumentTypeBuilder withErsteller(String ersteller) {
		this.ersteller = ersteller;
		return this;
	}

	public PrimaerdokumentTypeBuilder withCreatedAt(ZonedDateTime createdAt) {
		this.createdAt = createdAt;
		return this;
	}

	public PrimaerdokumentType build() {
		if (ozgFile == null) {
			throw new IllegalStateException("Can not build Primaerdokument when ozg file is null");
		}

		var primaerdokument = new PrimaerdokumentType();
		primaerdokument.setDateiname(ExportFilenameGenerator.generateExportFilename(ozgFile));
		primaerdokument.setDateinameOriginal(ozgFile.getName());
		primaerdokument.setErsteller(ersteller);
		if (createdAt != null) {
			primaerdokument.setDatumUhrzeit(DateConverter.toXmlGregorianCalendar(createdAt));
		} else {
			primaerdokument.setDatumUhrzeit(DateConverter.createEmpty());
		}
		return primaerdokument;
	}
}
