package de.ozgcloud.alfa.kommentar;

import java.util.Collections;
import java.util.List;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.alfa.common.PrimaerdokumentTypeBuilder;
import de.ozgcloud.alfa.common.UUIDConverter;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.xoev.xdomea.AnlageDokumentType;
import de.xoev.xdomea.DateiformatCodeType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.FormatType;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.VersionType;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class DokumentTypeBuilder {

	static final String VERSION_NUMMER = "1";
	static final String TYP = "Notiz";
	static final String DATEI_FORMAT_LIST_URI = "urn:xoev-de:xdomea:codeliste:dateiformat";

	private Kommentar kommentar;
	private List<OzgFile> kommentarAttachments = Collections.emptyList();

	public static DokumentTypeBuilder builder() {
		return new DokumentTypeBuilder();
	}

	public DokumentTypeBuilder withKommentar(Kommentar kommentar) {
		this.kommentar = kommentar;
		return this;
	}

	public DokumentTypeBuilder withKommentarAttachments(List<OzgFile> kommentarAttachments) {
		this.kommentarAttachments = kommentarAttachments;
		return this;
	}

	public DokumentType build() {
		var dokumentType = new DokumentType();
		dokumentType.setIdentifikation(createKommentarIdentifikation());
		dokumentType.setTyp(TYP);
		kommentarAttachments.stream().map(this::createAnlage).forEach(dokumentType.getAnlage()::add);
		return dokumentType;
	}

	IdentifikationObjektType createKommentarIdentifikation() {
		var identifikation = new IdentifikationObjektType();
		identifikation.setID(UUIDConverter.fromObjectId(kommentar.getId()));
		return identifikation;
	}

	AnlageDokumentType createAnlage(OzgFile ozgFile) {
		var anlage = new AnlageDokumentType();
		anlage.setIdentifikation(createKommentarAttachmentIdentifikation(ozgFile));
		anlage.getVersion().add(createVersionType(ozgFile));
		return anlage;
	}

	IdentifikationObjektType createKommentarAttachmentIdentifikation(OzgFile attachment) {
		var identifikation = new IdentifikationObjektType();
		identifikation.setID(UUIDConverter.fromObjectId(attachment.getId().toString()));
		return identifikation;
	}

	VersionType createVersionType(OzgFile ozgFile) {
		var versionType = new VersionType();
		versionType.setNummer(VERSION_NUMMER);
		versionType.getFormat().add(createFormatType(ozgFile));
		return versionType;
	}

	FormatType createFormatType(OzgFile ozgFile) {
		var formatType = new FormatType();
		formatType.setName(createDateiformatCodeType(ozgFile));
		formatType.setVersion(StringUtils.EMPTY);
		formatType.setPrimaerdokument(PrimaerdokumentTypeBuilder.builder()
				.withOzgFile(ozgFile)
				.build());
		return formatType;
	}

	DateiformatCodeType createDateiformatCodeType(OzgFile ozgFile) {
		var dateiformatCode = new DateiformatCodeType();
		dateiformatCode.setCode(DateiformatCode.getXdomeaCode(ozgFile.getContentType(), FilenameUtils.getExtension(ozgFile.getName())));
		dateiformatCode.setListURI(DATEI_FORMAT_LIST_URI);
		return dateiformatCode;
	}

}
