package de.ozgcloud.alfa.vorgang;

import java.time.ZoneOffset;
import java.time.ZonedDateTime;

import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.DateConverter;
import de.ozgcloud.alfa.export.XdomeaProperties;
import de.xoev.xdomea.BehoerdenkennungType;
import de.xoev.xdomea.Code;
import de.xoev.xdomea.KontaktType;
import de.xoev.xdomea.NachrichtentypCodeType;
import de.xoev.xdomea.NkAbgabeType;
import de.xoev.xdomea.OrganisationseinheitType;
import de.xoev.xdomea.SystemType;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
class KopfCreator {

	static final String BEHOERDENSCHLUSSEL_LIST_URI = "urn:de:bund:destatis:bevoelkerungsstatistik:schluessel:rs";
	static final String BEHOERDENSCHLUSSEL_LIST_VERSION_ID = "2023-11-30";

	static final String NACHRICHTENTYP_CODE_TYPE_LIST_URI = "urn:xoev-de:xdomea:codeliste:nachrichtentyp";
	static final String NACHRICHTENTYP_CODE_TYPE_LIST_VERSION_ID = "2.0";
	static final String NACHRICHTENTYP_ABGABE_ABGABE_TYPE_CODE = "0401";
	static final String PRODUKT_NAME = "OZG-Cloud";

	private final XdomeaProperties xDomeaProperties;

	public NkAbgabeType createKopf(VorgangWithEingang vorgang) {
		var nkAbgabeType = new NkAbgabeType();
		nkAbgabeType.setProzessID(vorgang.getId());
		nkAbgabeType.setNachrichtentyp(createNachrichtentyp());
		nkAbgabeType.setErstellungszeitpunkt(DateConverter.toXmlGregorianCalendar(ZonedDateTime.now(ZoneOffset.UTC)));
		nkAbgabeType.setAbsender(createAbsender(vorgang.getEingang().getZustaendigeStelle().getOrganisationseinheitenId()));
		nkAbgabeType.setEmpfaenger(createKontaktType());
		nkAbgabeType.setSendendesSystem(creeateSendendesSystem());
		nkAbgabeType.setImportbestaetigung(true);
		nkAbgabeType.setEmpfangsbestaetigung(true);
		return nkAbgabeType;
	}

	NachrichtentypCodeType createNachrichtentyp() {
		var nachrichtentypCode = new NachrichtentypCodeType();
		nachrichtentypCode.setCode(NACHRICHTENTYP_ABGABE_ABGABE_TYPE_CODE);
		nachrichtentypCode.setListURI(NACHRICHTENTYP_CODE_TYPE_LIST_URI);
		nachrichtentypCode.setListVersionID(NACHRICHTENTYP_CODE_TYPE_LIST_VERSION_ID);
		return nachrichtentypCode;
	}

	KontaktType createAbsender(String organisationseinheitId) {
		var absender = createKontaktType();
		absender.setOrganisationseinheit(createOrganisationseinheitType(organisationseinheitId));
		return absender;
	}

	KontaktType createKontaktType() {
		var kontakt = new KontaktType();
		kontakt.setBehoerdenkennung(createBehoerdenkennung());
		return kontakt;
	}

	OrganisationseinheitType createOrganisationseinheitType(String organisationseinheitId) {
		var organisationseinheit = new OrganisationseinheitType();
		organisationseinheit.setName(organisationseinheitId);
		return organisationseinheit;
	}

	BehoerdenkennungType createBehoerdenkennung() {
		var behoerdenkennungType = new BehoerdenkennungType();
		behoerdenkennungType.setBehoerdenschluessel(createBehoerdenschlussen());
		return behoerdenkennungType;
	}

	Code createBehoerdenschlussen() {
		var behoerdenschlussel = new Code();
		behoerdenschlussel.setCode(xDomeaProperties.getBehoerdenschluessel());
		behoerdenschlussel.setListURI(BEHOERDENSCHLUSSEL_LIST_URI);
		behoerdenschlussel.setListVersionID(BEHOERDENSCHLUSSEL_LIST_VERSION_ID);
		return behoerdenschlussel;
	}

	SystemType creeateSendendesSystem() {
		var sendendesSystem = new SystemType();
		sendendesSystem.setProduktname(PRODUKT_NAME);
		return sendendesSystem;
	}

}
