package de.ozgcloud.alfa.export;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.ByteArrayOutputStream;
import java.nio.charset.StandardCharsets;
import java.util.HexFormat;
import java.util.UUID;
import java.util.stream.Stream;

import org.apache.commons.lang3.RandomStringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;

import de.ozgcloud.alfa.common.binaryfile.FileId;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.file.ExportFileService;
import de.ozgcloud.alfa.historie.ExportHistorieService;
import de.ozgcloud.alfa.kommentar.ExportKommentarService;
import de.ozgcloud.alfa.vorgang.ExportVorgangService;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.common.test.ITCase;

@ITCase
class ExportServiceITCase {

	@SpyBean
	private ExportFileService exportFileService;
	@SpyBean
	private ExportVorgangService exportVorgangService;
	@MockBean
	private ExportHistorieService exportHistorieService;
	@MockBean
	private ExportKommentarService exportKommentarService;
	@Autowired
	private ExportService exportService;

	@Nested
	class TestWriteExport {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@BeforeEach
		void setup() {
			doReturn(vorgang).when(exportVorgangService).getVorgang(VorgangHeaderTestFactory.ID);
			doReturn(Stream.of(createOzgFile())).when(exportFileService).getRepresentations(vorgang);
			doReturn(Stream.of(createOzgFile())).when(exportFileService).getAttachments(vorgang);
			doNothing().when(exportFileService).writeOzgFile(any(), any());
			when(exportHistorieService.createHistorienProtokollInformationTypes(vorgang)).thenReturn(Stream.empty());
			when(exportKommentarService.createDokumentTypes(vorgang)).thenReturn(Stream.empty());
		}

		@Test
		void shouldNotThrowException() {
			assertDoesNotThrow(
					() -> exportService.writeExport(VorgangHeaderTestFactory.ID, UUID.randomUUID().toString(), new ByteArrayOutputStream()));
		}
	}

	private static OzgFile createOzgFile() {
		return OzgFileTestFactory.createBuilder()
				.id(FileId.from(createMongoDbObjectId()))
				.build();
	}

	private static String createMongoDbObjectId() {
		return HexFormat.of().formatHex(RandomStringUtils.randomAlphanumeric(24).getBytes(StandardCharsets.UTF_8));
	}
}
