package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;

import java.util.List;

import org.junit.jupiter.api.Test;

import de.ozgcloud.alfa.vorgang.VorgangTypeTestFactory;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.HistorienProtokollInformationType;
import de.xoev.xdomea.NkAbgabeType;
import de.xoev.xdomea.VorgangType;

class XdomeaNachrichtBuilderTest {

	private final NkAbgabeType kopfType = NkAbgabeTypeTestFactory.create();
	private final VorgangType vorgangType = VorgangTypeTestFactory.create();
	private final List<DokumentType> representations = List.of(DokumentTypeTestFactory.create(), DokumentTypeTestFactory.create());
	private final List<DokumentType> attachments = List.of(DokumentTypeTestFactory.create(), DokumentTypeTestFactory.create());
	private final List<HistorienProtokollInformationType> historie = List.of(HistorienProtokollInformationTypeTestFactory.create(),
			HistorienProtokollInformationTypeTestFactory.create());
	private final List<DokumentType> kommentare = List.of(DokumentTypeTestFactory.create(), DokumentTypeTestFactory.create());
	private final XdomeaNachrichtBuilder builder = XdomeaNachrichtBuilder.builder().withVorgang(vorgangType);

	@Test
	void shoulAddOneSchriftgutobjekt() {
		var abgabeType = builder.build();

		assertThat(abgabeType.getSchriftgutobjekt()).size().isEqualTo(1);
	}

	@Test
	void shouldSetVorgang() {
		var abgabeType = builder.build();

		assertThat(abgabeType.getSchriftgutobjekt().get(0).getVorgang()).isEqualTo(vorgangType);
	}

	@Test
	void shouldSetKopf() {
		var abgabeType = builder.withKopf(kopfType).build();

		assertThat(abgabeType.getKopf()).isEqualTo(kopfType);
	}

	@Test
	void shouldAddRepresentations() {
		var abgabeType = builder.withRepresentations(representations).build();

		assertThat(abgabeType.getSchriftgutobjekt().get(0).getVorgang().getDokument()).isEqualTo(representations);
	}

	@Test
	void shouldAddAttachments() {
		var abgabeType = builder.withAttachments(attachments).build();

		assertThat(abgabeType.getSchriftgutobjekt().get(0).getVorgang().getDokument()).isEqualTo(attachments);
	}

	@Test
	void shouldAddHistorie() {
		var abgabeType = builder.withHistorie(historie).build();

		assertThat(abgabeType.getSchriftgutobjekt().get(0).getVorgang().getHistorienProtokollInformation()).isEqualTo(historie);
	}

	@Test
	void shouldNotAddHistorie() {
		var abgabeType = builder.build();

		assertThat(abgabeType.getSchriftgutobjekt().get(0).getVorgang().getHistorienProtokollInformation()).isEmpty();
	}

	@Test
	void shouldAddKommentare() {
		var abgabeType = builder.withKommentare(kommentare).build();

		assertThat(abgabeType.getSchriftgutobjekt().get(0).getVorgang().getDokument()).isEqualTo(kommentare);
	}
}
