package de.ozgcloud.alfa.historie;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.common.DateConverter;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.export.HistorienProtokollInformationTypeTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.xoev.xdomea.HistorienProtokollInformationType;

class ExportHistorieServiceTest {

	@Spy
	@InjectMocks
	private ExportHistorieService service;

	@Mock
	private VorgangChangeHistoryService vorgangChangeHistoryService;

	@Nested
	class TestCreateHistorienProtokollInformationTypes {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final VorgangChangeHistory history = VorgangChangeHistoryTestFactory.create();
		private final HistorienProtokollInformationType statusChangeHistorienProtokollInformationType = HistorienProtokollInformationTypeTestFactory.create();
		private final HistorienProtokollInformationType aktenzeichenChangeHistorienProtokollInformationType = HistorienProtokollInformationTypeTestFactory.create();
		private final HistorienProtokollInformationType assignUserChangeHistorienProtokollInformationType = HistorienProtokollInformationTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			when(vorgangChangeHistoryService.createVorgangChangeHistory(vorgang)).thenReturn(history);
			doReturn(statusChangeHistorienProtokollInformationType).when(service)
					.createHistorienProtokollInformationType(VorgangChangeHistoryTestFactory.STATUS_CHANGE_HISTORY.get(0));
			doReturn(aktenzeichenChangeHistorienProtokollInformationType).when(service)
					.createHistorienProtokollInformationType(VorgangChangeHistoryTestFactory.AKTENZEICHEN_CHANGE_HISTORY.get(0));
			doReturn(assignUserChangeHistorienProtokollInformationType).when(service)
					.createHistorienProtokollInformationType(VorgangChangeHistoryTestFactory.ASSIGNED_USER_CHANGE_HISTORY.get(0));
		}

		@Test
		void shouldGetVorgangChangeHistory() {
			service.createHistorienProtokollInformationTypes(vorgang).toList();

			verify(vorgangChangeHistoryService).createVorgangChangeHistory(vorgang);
		}

		@Test
		void shouldCreateStatusChangeHistory() {
			service.createHistorienProtokollInformationTypes(vorgang).toList();

			verify(service).createHistorienProtokollInformationType(VorgangChangeHistoryTestFactory.STATUS_CHANGE_HISTORY.get(0));
		}

		@Test
		void shouldReturnStatusChangeHistorienProtokollInformationType() {
			var historienProtokollInformationTypes = service.createHistorienProtokollInformationTypes(vorgang);

			assertThat(historienProtokollInformationTypes).contains(statusChangeHistorienProtokollInformationType);
		}

		@Test
		void shouldCreateAktenzeichenChangeHistory() {
			service.createHistorienProtokollInformationTypes(vorgang).toList();

			verify(service).createHistorienProtokollInformationType(VorgangChangeHistoryTestFactory.AKTENZEICHEN_CHANGE_HISTORY.get(0));
		}

		@Test
		void shouldReturnAktenzeichenHistorienProtokollInformationType() {
			var historienProtokollInformationTypes = service.createHistorienProtokollInformationTypes(vorgang);

			assertThat(historienProtokollInformationTypes).contains(aktenzeichenChangeHistorienProtokollInformationType);
		}

		@Test
		void shouldCreateAssignUserChangeHistory() {
			service.createHistorienProtokollInformationTypes(vorgang).toList();

			verify(service).createHistorienProtokollInformationType(VorgangChangeHistoryTestFactory.ASSIGNED_USER_CHANGE_HISTORY.get(0));
		}

		@Test
		void shouldReturnAssignUserHistorienProtokollInformationType() {
			var historienProtokollInformationTypes = service.createHistorienProtokollInformationTypes(vorgang);

			assertThat(historienProtokollInformationTypes).contains(assignUserChangeHistorienProtokollInformationType);
		}
	}

	@Nested
	class TestCreateHistorienProtokollInformationType {

		private static final String CREATED_VALUE_BEFORE_CHANGE =
				VorgangChangeTestFactory.VALUE_BEFORE_CHANGE + VorgangChangeTestFactory.ORGANISATIONSEINHEITEN_ID;
		private static final String CREATED_VALUE_AFTER_CHANGE =
				VorgangChangeTestFactory.VALUE_AFTER_CHANGE + VorgangChangeTestFactory.ORGANISATIONSEINHEITEN_ID;
		private final VorgangChange vorgangChange = VorgangChangeTestFactory.create();

		@BeforeEach
		void setUp() {
			doReturn(VorgangChangeTestFactory.CREATED_BY_NAME).when(service).createAkteur(vorgangChange);
			doReturn(CREATED_VALUE_BEFORE_CHANGE).when(service).createValueBeforeChange(vorgangChange);
			doReturn(CREATED_VALUE_AFTER_CHANGE).when(service).createValueAfterChange(vorgangChange);
		}

		@Test
		void shouldCreateValueBeforeChange() {
			service.createHistorienProtokollInformationType(vorgangChange);

			verify(service).createValueBeforeChange(vorgangChange);
		}

		@Test
		void shouldHaveAlterWert() {
			var created = service.createHistorienProtokollInformationType(vorgangChange);

			assertThat(created.getMetadatumAlterWert()).isEqualTo(CREATED_VALUE_BEFORE_CHANGE);
		}

		@Test
		void shouldCreateValueAfterChange() {
			service.createHistorienProtokollInformationType(vorgangChange);

			verify(service).createValueAfterChange(vorgangChange);
		}

		@Test
		void shouldHaveNeuerWert() {
			var created = service.createHistorienProtokollInformationType(vorgangChange);

			assertThat(created.getMetadatumNeuerWert()).isEqualTo(CREATED_VALUE_AFTER_CHANGE);
		}

		@Test
		void shouldHaveAkteur() {
			var created = service.createHistorienProtokollInformationType(vorgangChange);

			assertThat(created.getAkteur()).isEqualTo(VorgangChangeTestFactory.CREATED_BY_NAME);
		}

		@Test
		void shouldHaveDatumUhrzeit() {
			var created = service.createHistorienProtokollInformationType(vorgangChange);

			assertThat(created.getDatumUhrzeit()).isEqualTo(DateConverter.toXmlGregorianCalendar(VorgangChangeTestFactory.FINISHED_AT));
		}

		@Test
		void shouldHaveAktion() {
			var created = service.createHistorienProtokollInformationType(vorgangChange);

			assertThat(created.getAktion()).isEqualTo(VorgangChangeTestFactory.ORDER.name());
		}

		@Test
		void shouldCreateAkteur() {
			service.createHistorienProtokollInformationType(vorgangChange);

			verify(service).createAkteur(vorgangChange);
		}
	}

	@Nested
	class TestCreateAkteur {

		private static final String VALUE_FOR_AKTEUR = LoremIpsum.getInstance().getWords(2);

		@Test
		void shouldGetValueForAkteur() {
			service.createAkteur(VorgangChangeTestFactory.create());

			verify(service).appendOrganisationseinheitenID(VorgangChangeTestFactory.CREATED_BY_NAME,
					VorgangChangeTestFactory.ORGANISATIONSEINHEITEN_ID);
		}

		@Test
		void shouldReturnValueForAkteur() {
			doReturn(VALUE_FOR_AKTEUR).when(service)
					.appendOrganisationseinheitenID(VorgangChangeTestFactory.CREATED_BY_NAME, VorgangChangeTestFactory.ORGANISATIONSEINHEITEN_ID);

			var akteur = service.createAkteur(VorgangChangeTestFactory.create());

			assertThat(akteur).isEqualTo(VALUE_FOR_AKTEUR);
		}
	}

	@Nested
	class TestCreateValueBeforeChange {

		private static final String ASSIGNED_USER = LoremIpsum.getInstance().getWords(2);

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "ASSIGN_USER")
		void shouldReturnValueForOrder(CommandOrder order) {
			var value = service.createValueBeforeChange(VorgangChangeTestFactory.createBuilder().order(order).build());

			assertThat(value).isEqualTo(VorgangChangeTestFactory.VALUE_BEFORE_CHANGE);
		}

		@Test
		void shouldGetValueForAssignUserOrder() {
			service.createValueBeforeChange(VorgangChangeTestFactory.createBuilder().order(CommandOrder.ASSIGN_USER).build());

			verify(service).appendOrganisationseinheitenID(VorgangChangeTestFactory.VALUE_BEFORE_CHANGE,
					VorgangChangeTestFactory.ORGANISATIONSEINHEITEN_ID);
		}

		@Test
		void shouldReturnValueForAssignUserOrder() {
			doReturn(ASSIGNED_USER).when(service)
					.appendOrganisationseinheitenID(VorgangChangeTestFactory.VALUE_BEFORE_CHANGE, VorgangChangeTestFactory.ORGANISATIONSEINHEITEN_ID);

			var value = service.createValueBeforeChange(VorgangChangeTestFactory.createBuilder().order(CommandOrder.ASSIGN_USER).build());

			assertThat(value).isEqualTo(ASSIGNED_USER);
		}
	}

	@Nested
	class TestCreateValueAfterChange {

		private static final String ASSIGNED_USER = LoremIpsum.getInstance().getWords(2);

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "ASSIGN_USER")
		void shouldReturnValueForOrder(CommandOrder order) {
			var value = service.createValueAfterChange(VorgangChangeTestFactory.createBuilder().order(order).build());

			assertThat(value).isEqualTo(VorgangChangeTestFactory.VALUE_AFTER_CHANGE);
		}

		@Test
		void shouldGetValueForAssignUserOrder() {
			service.createValueAfterChange(VorgangChangeTestFactory.createBuilder().order(CommandOrder.ASSIGN_USER).build());

			verify(service).appendOrganisationseinheitenID(VorgangChangeTestFactory.VALUE_AFTER_CHANGE,
					VorgangChangeTestFactory.ORGANISATIONSEINHEITEN_ID);
		}

		@Test
		void shouldReturnValueForAssignUserOrder() {
			doReturn(ASSIGNED_USER).when(service)
					.appendOrganisationseinheitenID(VorgangChangeTestFactory.VALUE_AFTER_CHANGE, VorgangChangeTestFactory.ORGANISATIONSEINHEITEN_ID);

			var value = service.createValueAfterChange(VorgangChangeTestFactory.createBuilder().order(CommandOrder.ASSIGN_USER).build());

			assertThat(value).isEqualTo(ASSIGNED_USER);
		}
	}

	@Nested
	class TestAppendOrganisationseinheitenID {

		private static final String TEXT = LoremIpsum.getInstance().getWords(2);

		@ParameterizedTest
		@NullAndEmptySource
		void shouldReturnEmpty(String assignedUser) {
			var value = service.appendOrganisationseinheitenID(assignedUser, VorgangChangeTestFactory.ORGANISATIONSEINHEITEN_ID);

			assertThat(value).isEmpty();
		}

		@Test
		void shouldAppendOrganisationseinheitenID() {
			var value = service.appendOrganisationseinheitenID(TEXT, VorgangChangeTestFactory.ORGANISATIONSEINHEITEN_ID);

			assertThat(value).isEqualTo(
					String.format("%s; %s", TEXT, VorgangChangeTestFactory.ORGANISATIONSEINHEITEN_ID));
		}
	}

}