package de.ozgcloud.alfa.kommentar;

import static de.ozgcloud.alfa.common.TestUtils.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.stream.Stream;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileService;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.export.DokumentTypeTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.xoev.xdomea.DokumentType;

class ExportKommentarServiceTest {

	@InjectMocks
	private ExportKommentarService service;

	@Mock
	private KommentarService kommentarService;
	@Mock
	private BinaryFileService binaryFileService;

	@Nested
	class TestCreateDokumentTypes {

		private MockedStatic<DokumentTypeBuilder> dokumentTypeBuilderMockedStatic;

		private final Kommentar kommentar = KommentarTestFactory.create();
		private final List<OzgFile> ozgFiles = List.of(OzgFileTestFactory.create());
		private final DokumentType dokumentType = DokumentTypeTestFactory.create();

		@Mock
		private DokumentTypeBuilder dokumentTypeBuilder;

		@BeforeEach
		void setUp() {
			when(kommentarService.findByVorgangId(VorgangHeaderTestFactory.ID)).thenReturn(Stream.of(kommentar));

			dokumentTypeBuilderMockedStatic = mockStatic(DokumentTypeBuilder.class);
			dokumentTypeBuilderMockedStatic.when(DokumentTypeBuilder::builder).thenReturn(dokumentTypeBuilder);
			when(dokumentTypeBuilder.withKommentarAttachments(ozgFiles)).thenReturn(dokumentTypeBuilder);
			when(dokumentTypeBuilder.withKommentar(kommentar)).thenReturn(dokumentTypeBuilder);
			when(dokumentTypeBuilder.build()).thenReturn(dokumentType);

			mockStreamToList(ozgFiles, stream -> when(binaryFileService.getFiles(KommentarTestFactory.ATTACHMENTS)).thenReturn(stream));
		}

		@AfterEach
		void tearDown() {
			dokumentTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldLoadKommentare() {
			service.createDokumentTypes(VorgangWithEingangTestFactory.create()).toList();

			verify(kommentarService).findByVorgangId(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldLoadOzgFilesForKommentareAttachments() {
			service.createDokumentTypes(VorgangWithEingangTestFactory.create()).toList();

			verify(binaryFileService).getFiles(KommentarTestFactory.ATTACHMENTS);
		}

		@Test
		void shouldBuildDokumentType() {
			service.createDokumentTypes(VorgangWithEingangTestFactory.create()).toList();

			verify(dokumentTypeBuilder).build();
		}

		@Test
		void shouldSetOzgFile() {
			service.createDokumentTypes(VorgangWithEingangTestFactory.create()).toList();

			verify(dokumentTypeBuilder).withKommentarAttachments(ozgFiles);
		}

		@Test
		void shouldSetKommentar() {
			service.createDokumentTypes(VorgangWithEingangTestFactory.create()).toList();

			verify(dokumentTypeBuilder).withKommentar(kommentar);
		}
	}

}